#pragma once
#ifndef _ZMID520X_ONESTEP_H_
#define _ZMID520X_ONESTEP_H_ 1

#define IN  
#define OUT  
#define INOUT 

#ifdef __cplusplus 
extern "C" {
#endif 

#if defined DLL_EXPORT
#define ZMID520X_ONESTEP_API __declspec(dllexport)
#else
#define ZMID520X_ONESTEP_API __declspec(dllimport)
#endif

/* Maximum number of spatial angle/Pos0 points that the DLL can process */
#define MAX_NR_POINTS 1500

/* ZMID520x types to be used with the RawToPercent and PercentToRaw functions*/
#define ZMID5201 1
#define ZMID5202 2
#define ZMID5203 3

/* Error codes for the calibration functions */
#define SUCCESS				0		//The function returned without errors
#define ERR_INVALID_INPUT	-1		//One or more of the inputs are invalid
#define ERR_SLOPE			-2		//The calculated slope is out of range
#define ERR_OFFSET			-3		//The calculated offset is out of range
#define ERR_CORR			-4		//One or more of the calculated correction values are out of range
#define ERR_POS0			-5		//One or more of the calculated Pos0 values are out of range
#define ERR_POS1			-6		//One or more of the calculated Pos1 values are out of range

/* Retrurns the version of the DLL library */
ZMID520X_ONESTEP_API double __stdcall ZMID520X_GetDllVersion();

/* Converts raw values [0 - 65535] to percent of output [0-100%] ( or [5 - 95%] for ZMID5201/2 ) */
ZMID520X_ONESTEP_API double __stdcall ZMID520X_RawToPercent(double raw, __int32 zmid_type);

/* Converts percent of output [0-100%] ( or [5 - 95%] for ZMID5201/2 ) to  raw values [0 - 65535] */
ZMID520X_ONESTEP_API double __stdcall ZMID520X_PercentToRaw(double percent, __int32 zmid_type);

/* Performs a one step calibration on an array of spatial angle values */
ZMID520X_ONESTEP_API __int32 __stdcall ZMID520X_OneStepCalibration(IN const __int32 zmid_type, IN const double spatialangle[],
	IN int size_spatialangle, IN double position_slope, IN double reverse_slope, IN double out_min, IN double out_max, IN double out_mod, 
	OUT double *offset, OUT double *offset_reg, OUT double *slope, OUT double *slope_reg, OUT double corr[], OUT double corr_reg[], 
	OUT double *clamp_low, OUT double *clamp_high, OUT double *clamp_reg);

/* Calculates Pos0 and Pos 1*/
ZMID520X_ONESTEP_API __int32 __stdcall ZMID520X_Pos0_Pos1(IN double out_mod, IN double position_slope,
	IN const double corr_val[], IN double offset_reg, IN double slope_reg, IN const double spatialangle[],
	IN int size_spatialangle, OUT double pos0[], OUT double pos1[]);

/* Calculates Offset and Slope */
ZMID520X_ONESTEP_API __int32 __stdcall ZMID520X_Offset_Slope(IN const double spatialangle[],
	IN int size_spatialangle, IN double position_slope, IN double reverse_slope, IN double out_min, IN double out_max, 
	IN double out_mod, OUT double *offset, OUT double *offset_reg, OUT double *slope, OUT double *slope_reg);

/* Calculates correction values */
ZMID520X_ONESTEP_API __int32 __stdcall ZMID520X_Correction(IN double pos0[], IN int size_spatialangle, IN double position_slope,
	IN double reverse_slope, IN double out_min, IN double out_max, OUT double corr[], OUT double corr_reg[]);

/* Calculates clamp values */
ZMID520X_ONESTEP_API __int32 __stdcall ZMID520X_Clamps(IN double clamp_low_value, IN double clamp_high_value,
	OUT double *clamp_low, OUT double *clamp_high, OUT double *clamp_reg);

ZMID520X_ONESTEP_API __int32 __stdcall ZMID520X_OneStepCalibration_Analog(IN const double analog_values[],
	IN int size_array, IN double vdd_value, IN double position_slope, IN double reverse_slope, IN double out_min, IN double out_max, IN double out_mod,
	OUT double *offset, OUT double *offset_reg, OUT double *slope, OUT double *slope_reg, OUT double corr[], OUT double corr_reg[],
	OUT double *clamp_low, OUT double *clamp_high, OUT double *clamp_reg);

ZMID520X_ONESTEP_API __int32 __stdcall ZMID520X_OneStepCalibration_PWM(IN const double pwm_values[],
	IN int size_array, IN double position_slope, IN double reverse_slope, IN double out_min, IN double out_max, IN double out_mod,
	OUT double *offset, OUT double *offset_reg, OUT double *slope, OUT double *slope_reg, OUT double corr[], OUT double corr_reg[],
	OUT double *clamp_low, OUT double *clamp_high, OUT double *clamp_reg);

ZMID520X_ONESTEP_API __int32 __stdcall ZMID520X_OneStepCalibration_SENT(IN const double sent_values[],
	IN int size_array, IN double position_slope, IN double reverse_slope, IN double out_min, IN double out_max, IN double out_mod,
	OUT double *offset, OUT double *offset_reg, OUT double *slope, OUT double *slope_reg, OUT double corr[], OUT double corr_reg[],
	OUT double *clamp_low, OUT double *clamp_high, OUT double *clamp_reg);

#ifdef __cplusplus
	} //extern "C"
#endif 

#endif